@@ -6,7 +6,7 @@
  * Released to the public domain.
  *
  *--------------------------------------------------------------------------
- * $Id: hashtable.c,v 9999.23 2018/05/05 00:47:52 cvs Exp $
+ * $Id: hashtable.c,v 9999.32 2019/01/31 03:15:09 cvs Exp $
 \*--------------------------------------------------------------------------*/
 
 #include <stdio.h>
@@ -23,6 +23,40 @@ static unsigned long pointerHashFunction(const void *pointer);
 static int isProbablePrime(long number);
 static long calculateIdealNumOfBuckets(HashTable *hashTable);
 
+long long_random_val(){
+	long ret = 0;
+	if(RAND_MAX<255){
+		SUBREADprintf("Is this a embedded computer????\n");
+		return -1;
+	}
+	int i;
+	for(i=0;i<8;i++){
+		if(i>0)ret = (ret << 8) ^ (myrand_rand() & 0xff);
+		else ret = (ret << 8) ^ (myrand_rand() & 0x7f);
+	}
+	return ret;
+}
+
+void * ArrayListShift(ArrayList * list){
+	if(list->numOfElements<1) return NULL;
+	void *ret = list->elementList [0];
+	long xx;
+	list->numOfElements -- ;
+	for(xx=0; xx<list->numOfElements; xx++) list->elementList [ xx ] = list->elementList [ xx+1 ];
+	return ret;
+}
+
+void * ArrayListPop(ArrayList * list){
+	if(list->numOfElements<1) return NULL;
+	return list->elementList [ -- list->numOfElements];
+}
+
+
+void * ArrayListRandom(ArrayList * list){
+	long ii = long_random_val() % list -> numOfElements;
+	return list -> elementList[ii];
+}
+
 
 ArrayList * ArrayListCreate(int init_capacity){
 	ArrayList * ret = malloc(sizeof(ArrayList));
@@ -130,6 +164,36 @@ void ArrayListSort(ArrayList * list, int compare_L_minus_R(void * L_elem, void *
 	merge_sort(sortdata, list -> numOfElements, ArrayListSort_compare, ArrayListSort_exchange, ArrayListSort_merge);
 }
 
+int ArrayListLLUComparison(void * L_elem, void * R_elem){
+	unsigned long long lint = L_elem-NULL;
+	unsigned long long rint = R_elem-NULL;
+	if(lint<rint)return -1;
+	if(lint>rint)return 1;
+	return 0;
+}
+
+long ArrayListFindNextDent(ArrayList * list, unsigned long long value_less_than_dent){
+	long h=list->numOfElements-1,l=0,m=-1l;
+	if( list -> elementList[h]- NULL <= value_less_than_dent )return -1l;
+	while(h>l){
+		m=(h+l)/2;
+		unsigned long long mv = list -> elementList[m] - NULL;
+		if(mv < value_less_than_dent) l=m+1;
+		else if(mv > value_less_than_dent) h=m-1;
+		else break;
+	}
+	if(m<2)m=0;
+	else m-=2;
+
+	for(; m>=0 && list -> elementList[m] - NULL >= value_less_than_dent; m--);
+
+	for(m = max(0, m); m < list->numOfElements ; m++){
+		if( list -> elementList[m] - NULL > value_less_than_dent )return m;
+	}
+	SUBREADprintf("ALGORITHM ERROR!! DID YOU SORT THE LIST???\n");
+	return -2l;
+}
+
 /*--------------------------------------------------------------------------*\
  *  NAME:
  *	  HashTableCreate() - creates a new HashTable
@@ -346,13 +410,13 @@ int HashTablePutReplaceEx(HashTable *hashTable, const void *key, void *value, in
 			if(replace_key) {
 				if(hashTable->keyDeallocator && dealloc_key)
 					hashTable->keyDeallocator((void *) pair->key);
-				pair->key = key;
+				pair->key = (void *)key;
 			}
 		}
 		if (pair->value != value) {
 			if (hashTable->valueDeallocator != NULL && dealloc_value)
 				hashTable->valueDeallocator(pair->value);
-			pair->value = value;
+			pair->value = (void *)value;
 		}
 	}
 	else {
@@ -361,8 +425,8 @@ int HashTablePutReplaceEx(HashTable *hashTable, const void *key, void *value, in
 		return -1;
 		}
 		else {
-			newPair->key = key;
-			newPair->value = value;
+			newPair->key = (void *)key;
+			newPair->value = (void *)value;
 			newPair->next = hashTable->bucketArray[hashValue];
 			hashTable->bucketArray[hashValue] = newPair;
 			hashTable->numOfElements++;
@@ -399,6 +463,18 @@ int HashTablePutReplace(HashTable *hashTable, const void *key, void *value, int
  *					 doesn't exist in the HashTable
 \*--------------------------------------------------------------------------*/
 
+void *HashTableGetKey(const HashTable *hashTable, const void *key) {
+	long hashValue = hashTable->hashFunction(key) % hashTable->numOfBuckets;
+
+	KeyValuePair *pair = hashTable->bucketArray[hashValue];
+
+	while (pair != NULL && hashTable->keycmp(key, pair->key) != 0)
+		pair = pair->next;
+
+	return (pair == NULL)? NULL : pair->key;
+}
+
+
 void *HashTableGet(const HashTable *hashTable, const void *key) {
 	long hashValue = hashTable->hashFunction(key) % hashTable->numOfBuckets;
 
