@@ -6,7 +6,7 @@
  * Released to the public domain.
  *
  *--------------------------------------------------------------------------
- * $Id: hashtable.h,v 9999.11 2017/04/28 01:40:20 cvs Exp $
+ * $Id: hashtable.h,v 9999.22 2019/01/31 03:15:09 cvs Exp $
 \*--------------------------------------------------------------------------*/
 
 #ifndef _HASHTABLE_H
@@ -16,7 +16,7 @@
  * All access should be via the public functions declared below. */
 
 typedef struct KeyValuePair_struct {
-    const void *key;
+    void *key;
     void *value;
     struct KeyValuePair_struct *next;
 } KeyValuePair;
@@ -49,14 +49,37 @@ typedef struct {
 } ArrayList;
 
 ArrayList * ArrayListCreate(int init_capacity);
+ArrayList * ArrayListDuplicate(ArrayList * ori);
 void ArrayListDestroy(ArrayList * list);
 void * ArrayListGet(ArrayList * list, long n);
+void * ArrayListRandom(ArrayList * list);
 int ArrayListPush(ArrayList * list, void * new_elem);
+int ArrayListPush_NoRepeatedPtr(ArrayList * list, void * new_elem);
+// Shift is the other direction of Pop:
+// Say, a list has elements 0,1,2,3,4
+// Pop removes 4 from the list and returns it.
+// Shift repoves 0 from the list and returns it.
+//
+// Caller has to be sure that at least one element is in the list or a NULL is returned. 
+// Due to its nature, the caller has to deallocate the returned object. The internal deallocater is NOT called on the returned object.
+void * ArrayListShift(ArrayList * list);
+void * ArrayListPop(ArrayList * list);
 void ArrayListSetDeallocationFunction(ArrayList * list,  void (*elem_deallocator)(void *elem));
 void ArrayListSort(ArrayList * list, int compare_L_minus_R(void * L_elem, void * R_elem));
 
+// A simple comparison function if you want to sort unsigned long long ints.
+int ArrayListLLUComparison(void * L_elem, void * R_elem);
+
+// This function returns the index of the value in the list that is the minumum value greater than "value_less_than_dent".
+// The elements in the list are all zero-based unsigned long long ints. The list must be sorted (small -> large) before calling this function.
+// If the "value_less_than_dent" is larger than all the values in the list, "-1" is returned;
+// If there are repeated numbers in the list, this function will always return the index of the first of them when this repeated number is the next dent.
+//
+long ArrayListFindNextDent(ArrayList * list, unsigned long long value_less_than_dent);
+
 
 void HashTableIteration(HashTable * tab, void process_item(void * key, void * hashed_obj, HashTable * tab) );
+ArrayList * HashTableKeyArray(HashTable * tab);
 
 /*--------------------------------------------------------------------------*\
  *  NAME:
@@ -85,6 +108,7 @@ void HashTableIteration(HashTable * tab, void process_item(void * key, void * ha
  *      HashTable    - a new Hashtable, or NULL on error
 \*--------------------------------------------------------------------------*/
 
+HashTable * StringTableCreate(long numOfBuckets);
 HashTable *HashTableCreate(long numOfBuckets);
 
 /*--------------------------------------------------------------------------*\
@@ -183,7 +207,8 @@ int HashTablePutReplace(HashTable *hashTable, const void *key, void *value, int
  *                     doesn't exist in the HashTable
 \*--------------------------------------------------------------------------*/
 
-void *HashTableGet(const HashTable *hashTable, const void *key);
+void * HashTableGetKey(const HashTable *hashTable, const void *key);
+void * HashTableGet(const HashTable *hashTable, const void *key);
 
 /*--------------------------------------------------------------------------*\
  *  NAME:
@@ -461,6 +486,8 @@ void HashTableSetDeallocationFunctions(HashTable *hashTable,
 unsigned long HashTableStringHashFunction(const void *key);
 
 void free_values_destroy(HashTable * tab);
+int HashTablePutReplaceEx(HashTable *hashTable, const void *key, void *value, int replace_key, int dealloc_key, int dealloc_value);
+
 #endif /* _HASHTABLE_H */
 
 
