@@ -31,6 +31,7 @@
 #include "HelperFunctions.h"
 #include "gene-algorithms.h"
 #include "sorted-hashtable.h"
+#include "seek-zlib.h"
 #include "input-files.h"
 
 #define NO_GENE_DEBUG_
@@ -44,6 +45,7 @@ int IS_COLOR_SPACE = 0;
 int VALUE_ARRAY_INDEX = 1;
 int MARK_NONINFORMATIVE_SUBREADS = 0;
 int IS_FORCED_ONE_BLOCK = 0;
+int ignore_bar_in_seqnames = 0;
 
 #define NEXT_READ 1
 #define NEXT_FILE 2
@@ -52,7 +54,7 @@ int IS_FORCED_ONE_BLOCK = 0;
 
 void print_build_log(double finished_rate, double read_per_second, double expected_seconds, unsigned long long int total_reads)
 {
-	print_in_box( 81,0,0,"%4d%%%%, %3d mins elapsed, rate=%.1fk bps/s, total=%llum\r", (int)(finished_rate*100), (int)(miltime()-begin_ftime)/60, read_per_second/1000 ,total_reads/1000000);
+	print_in_box( 81,0,0,"%4d%%%%, %3d mins elapsed, rate=%.1fk bps/s", (int)(finished_rate*100), (int)(miltime()-begin_ftime)/60, read_per_second/1000 );
 }
 
 void copy_non_informative_subread(gehash_t * index_table, gehash_t * noninf_table)
@@ -78,14 +80,13 @@ void copy_non_informative_subread(gehash_t * index_table, gehash_t * noninf_tabl
 
 #define MAX_BASES_IN_INDEX 4294900000.0
 
-int build_gene_index(const char index_prefix [], char ** chro_files, int chro_file_number, unsigned int memory_megabytes, int threshold, gehash_t * huge_table, unsigned int * chro_lens)
+int build_gene_index(const char index_prefix [], char ** chro_files, int chro_file_number, unsigned int memory_megabytes, int threshold, gehash_t * huge_table, unsigned int * chro_lens, long long actual_bases)
 {
 	int file_number, table_no;
 	int status = NEXT_FILE;
 	unsigned int offset, read_no;
 	unsigned int segment_size = (unsigned int)(memory_megabytes * 1024.0 / 8.) * 1024 ;
 	long long int all_bases = guess_gene_bases(chro_files,chro_file_number);
-	double local_begin_ftime = 0.;
 
 	int chro_table_maxsize=100, dump_res = 0;
 	unsigned int * read_offsets = malloc(sizeof(unsigned int) * chro_table_maxsize);
@@ -207,9 +208,6 @@ int build_gene_index(const char index_prefix [], char ** chro_files, int chro_fi
 
 				geinput_readline(&ginp, fn, 0);
 
-				if(offset==table.padding)
-					local_begin_ftime = miltime();
-
 				if(read_no>0){
 					read_offsets[read_no-1] = offset + table.padding;
 					offset += 2*table.padding;
@@ -217,7 +215,7 @@ int build_gene_index(const char index_prefix [], char ** chro_files, int chro_fi
 
 				//printf("TTTXT FN=%s\n",fn);
 
-				for(i=0;(fn[i+1] != ' ' && fn[i+1] != '\0' && fn[i+1] != '\t' && i<MAX_CHROMOSOME_NAME_LEN - 1); i++)
+				for(i=0;(fn[i+1] != ' ' && fn[i+1] != '\0' && fn[i+1] != '\t' && i<MAX_CHROMOSOME_NAME_LEN - 1 && ( fn[i+1] != '|'  || !ignore_bar_in_seqnames )); i++)
 					*(read_names + MAX_READ_NAME_LEN*read_no + i) = fn[i+1];
 
 				*(read_names + MAX_READ_NAME_LEN*read_no + i) = 0;
@@ -395,13 +393,13 @@ int build_gene_index(const char index_prefix [], char ** chro_files, int chro_fi
 					array_int_key = int_key;
 				}
 
-				if(all_bases > 12){
-					if (offset>0 && offset % (all_bases / 12) == 0)
+				if(actual_bases > 12){
+					if (offset>0 && offset % (actual_bases / 12) == 0)
 					{
-						double finished_rate = offset*1.0/all_bases;
-						double base_per_second = offset / (miltime()-local_begin_ftime);
-						double ETA = (all_bases-offset) / base_per_second;
-						print_build_log(finished_rate,base_per_second,ETA, all_bases);
+						double finished_rate = offset*1.0/actual_bases;
+						double base_per_second = offset / (miltime()-begin_ftime);
+						double ETA = (actual_bases-offset) / base_per_second;
+						print_build_log(finished_rate,base_per_second,ETA, actual_bases);
 						SUBREADfflush(stdout) ;
 					}
 				}
@@ -411,7 +409,7 @@ int build_gene_index(const char index_prefix [], char ** chro_files, int chro_fi
 
 				if(offset > 0xFFFFFFFDU)	
 				{
-					SUBREADprintf("ERROR: The chromosome data contains too many bases. The size of the input FASTA file(s) should be less than 4G Bytes\n") ;
+					SUBREADprintf("ERROR: the provided reference sequences include more than 4 billion bases.\n") ;
 					return -1;
 				}
 
@@ -470,13 +468,14 @@ int add_repeated_subread(gehash_t * tab , unsigned int subr, unsigned char ** hu
 }
 
 
-int scan_gene_index(const char index_prefix [], char ** chro_files, int chro_file_number, int threshold, gehash_t *huge_table)
+int scan_gene_index(const char index_prefix [], char ** chro_files, int chro_file_number, int threshold, gehash_t *huge_table, long long * actual_total_bases_inc_marging)
 {
 	int file_number, i ,j;
 	int status = NEXT_FILE;
 	unsigned int offset, read_no;
-	double local_begin_ftime = miltime();
-	long long int all_bases = guess_gene_bases(chro_files,chro_file_number);
+	long long int guess_all_bases = guess_gene_bases(chro_files,chro_file_number);
+	int padding_around_contigs = MAX_READ_LENGTH;
+	*actual_total_bases_inc_marging = padding_around_contigs;
 
 	gehash_t occurance_table;
 	unsigned char * huge_index[1024];
@@ -512,9 +511,9 @@ int scan_gene_index(const char index_prefix [], char ** chro_files, int chro_fil
 		SUBREADprintf("ERROR: The path is too long. It should not be longer than 290 chars.\n");
 		return -1;
 	}
-	if(all_bases<1)
+	if(guess_all_bases<1)
 	{
-		SUBREADprintf("ERROR: File '%s' is inaccessible.\n", chro_files[-all_bases-1]);
+		SUBREADprintf("ERROR: File '%s' is inaccessible.\n", chro_files[-guess_all_bases-1]);
 		return -1;
 	}
 
@@ -544,6 +543,7 @@ int scan_gene_index(const char index_prefix [], char ** chro_files, int chro_fil
 
 			if (status == NEXT_FILE)
 			{
+				(*actual_total_bases_inc_marging)+=padding_around_contigs;
 				if(file_number == chro_file_number)
 				{
 					geinput_close(&ginp);
@@ -560,6 +560,7 @@ int scan_gene_index(const char index_prefix [], char ** chro_files, int chro_fil
 			}
 			if (status == NEXT_READ)
 			{
+				(*actual_total_bases_inc_marging)+=2*padding_around_contigs;
 
 				geinput_readline(&ginp, fn, 0);
 
@@ -636,23 +637,12 @@ int scan_gene_index(const char index_prefix [], char ** chro_files, int chro_fil
 				}
 
 				offset ++;
+				(*actual_total_bases_inc_marging)++;
 				read_len ++;
 
-
-				if (all_bases>12){
-					if (offset % (all_bases / 12) == 0)
-					{
-						double finished_rate = offset*1.0/all_bases;
-						double base_per_second = offset / (miltime()-local_begin_ftime);
-						double ETA = (all_bases-offset) / base_per_second;
-						print_build_log(finished_rate,base_per_second,ETA, all_bases);
-						SUBREADfflush(stdout) ;
-					}
-				}
-
 				if(offset > 0xFFFFFFFDU)	
 				{
-					SUBREADprintf("ERROR: The chromosome data contains too many bases. The size of the input FASTA files should be less than 4G Bytes\n") ;
+					SUBREADprintf("ERROR: the provided reference sequences include more than 4 billion bases.\n") ;
 					return -1;
 				}
 
@@ -704,10 +694,20 @@ char *rtrim(char *s)
 	return s;
 }
 
+typedef struct{
+	char * filename;
+	unsigned int line_no;
+	int byte_in_line;
+} format_check_context_t;
+
 int ERROR_FOUND_IN_FASTA = 0;
-#define CHAR_ESC 27
-void check_and_convert_warn(char * FN, long long int fpos_line_head, unsigned line_no, int line_pos, char * msg, FILE * log_fp)
+void check_and_convert_warn(format_check_context_t * fcc, char * msg, FILE * log_fp){
+	ERROR_FOUND_IN_FASTA += 1;
+	fprintf(log_fp, "A format issue is found in the %u-th line in file '%s': %s.\n", fcc->line_no, fcc->filename, msg);
+}
+void check_and_convert_warnOLD(char * FN, long long int fpos_line_head, unsigned line_no, int line_pos, char * msg, FILE * log_fp)
 {
+#define CHAR_ESC 27
 	int x1,brs=0;
 	long long int back_search_ptr;
 	char * line_buf = malloc(MAX_READ_LENGTH+1);
@@ -799,6 +799,8 @@ int check_and_convert_FastA(char ** input_fas, int fa_number, char * out_fa, uns
 	int chrom_lens_len = 0;
 	ERROR_FOUND_IN_FASTA = 0;
 	FILE * out_fp = f_subr_open(out_fa,"w");
+	format_check_context_t fcc;
+	memset(&fcc,0,sizeof(fcc));
 
 	if(!out_fp)
 	{
@@ -818,12 +820,12 @@ int check_and_convert_FastA(char ** input_fas, int fa_number, char * out_fa, uns
 	print_in_box( 80,0,0,"Check the integrity of provided reference sequences ...");
 	for(inp_file_no = 0; inp_file_no < fa_number; inp_file_no++)
 	{
-		FILE * in_fp = f_subr_open(input_fas[inp_file_no],"r");
-		long long int last_read_head_pos = 0;
-		unsigned int last_read_line_no = 1;
+		autozip_fp fafp;
+		memset(&fafp, 0, sizeof(fafp));
+		int faret = autozip_open(input_fas[inp_file_no], &fafp);
+		fcc.filename = input_fas[inp_file_no];
 
-		if(!in_fp)
-		{
+		if(faret <0) {
 			SUBREADprintf("ERROR: Input file '%s' is not found or is not accessible. No index was built.\n", input_fas[inp_file_no]);
 			HashTableDestroy(rep_name_table);
 			return -1;
@@ -832,13 +834,17 @@ int check_and_convert_FastA(char ** input_fas, int fa_number, char * out_fa, uns
 		line_no = 0;
 		int is_head_written=0;
 		read_head_buf[0]=0;
-		while(!feof(in_fp))
-		{
-			long long int line_head_pos = ftello(in_fp);
+		while(1) {
 			unsigned int read_len = 0;
-			char * ret = fgets(line_buf, MAX_READ_LENGTH-1 , in_fp);
-			if(!ret) break;
+			int ret = autozip_gets(&fafp, line_buf, MAX_READ_LENGTH);
+			if(ret<1) break;
+			if(ret >= MAX_READ_LENGTH-2){
+				SUBREADprintf("ERROR: A fasta file cannot have a line longer than 1000 bytes. You need to split a very long line into many lines.\n");
+				return -1;
+			}
+
 			line_no ++;
+			fcc.line_no = line_no;
 			int line_buf_len = strlen(line_buf);
 
 			for(; line_buf[line_buf_len-1] == '\r' || line_buf[line_buf_len-1]=='\n' ;line_buf_len--)
@@ -848,7 +854,8 @@ int check_and_convert_FastA(char ** input_fas, int fa_number, char * out_fa, uns
 					if(!is_R_warnned)
 					{
 						is_R_warnned=1;
-						check_and_convert_warn(input_fas[inp_file_no], line_head_pos, line_no, line_buf_len -1 ,"This line ends with '\\r\\n'. It is not a problem for building the index but we suggest to use Unix-styled line breaks.", log_fp);
+						fcc.byte_in_line = line_buf_len-1;
+						check_and_convert_warn(&fcc ,"This line ends with '\\r\\n'. It is not a problem for building the index but we suggest to use Unix-styled line breaks.", log_fp);
 					}	
 				}
 				line_buf[line_buf_len-1] =0;
@@ -857,7 +864,8 @@ int check_and_convert_FastA(char ** input_fas, int fa_number, char * out_fa, uns
 
 			if(line_buf_len<1)
 			{
-				check_and_convert_warn(input_fas[inp_file_no], line_head_pos, line_no ,0 ,"This line is empty. This is not allowed in the FASTA file.", log_fp);
+				fcc.byte_in_line = -1;
+				check_and_convert_warn(&fcc ,"This line is empty. This is not allowed in the FASTA file.", log_fp);
 				continue;
 			}
 
@@ -865,11 +873,10 @@ int check_and_convert_FastA(char ** input_fas, int fa_number, char * out_fa, uns
 			{
 				if(line_no>1 &&!is_head_written)
 				{
-					check_and_convert_warn(input_fas[inp_file_no], last_read_head_pos, last_read_line_no, 0,"This sequence has less than 16 bases. It is ignored in the index because no subreads can be extracted.", log_fp);
+					fcc.byte_in_line = 0;
+					check_and_convert_warn(&fcc,"This sequence has less than 16 bases. It is ignored in the index because no subreads can be extracted.", log_fp);
 				}
 				is_head_written = 0;
-				last_read_line_no = line_no;
-				last_read_head_pos = line_head_pos;
 				read_len = 0;
 				read_head_buf[0]=0;
 
@@ -897,9 +904,10 @@ int check_and_convert_FastA(char ** input_fas, int fa_number, char * out_fa, uns
 				HashTablePut(rep_name_table, keymem, NULL+1);
 
 			}
-			else if(line_head_pos<1)
+			else if(line_no<1)
 			{
-				check_and_convert_warn(input_fas[inp_file_no], 0, 0, 0 ,"This file is not started with a header line. It seems not to be a FASTA file.", log_fp);
+				fcc.byte_in_line=0;
+				check_and_convert_warn(&fcc ,"This file is not started with a header line. It seems not to be a FASTA file.", log_fp);
 			}
 			else
 			{
@@ -910,7 +918,8 @@ int check_and_convert_FastA(char ** input_fas, int fa_number, char * out_fa, uns
 					int lowerch = tolower(nextch);
 					if(!( lowerch == 'a' || lowerch == 't' || lowerch == 'g' || lowerch == 'c' || nextch == '.' || nextch=='-' || lowerch=='n'))
 					{	
-						check_and_convert_warn(input_fas[inp_file_no], line_head_pos, line_no, xk2, "The pointed base was converted to an 'A'.", log_fp);
+						fcc.byte_in_line=xk2;
+						check_and_convert_warn(&fcc, "The non-ACGT base was converted to an 'A'.", log_fp);
 						line_buf[xk2] = 'A';
 					}
 					else if(nextch == '.' || nextch=='-' ||  lowerch=='n')
@@ -956,7 +965,7 @@ int check_and_convert_FastA(char ** input_fas, int fa_number, char * out_fa, uns
 			}
 		}
 
-		fclose(in_fp);
+		autozip_close(&fafp);
 		if(is_disk_full) break;
 	}
 
@@ -1031,9 +1040,12 @@ int main_buildindex(int argc,char ** argv)
 
 	optind = 0;
 	
-	while ((c = getopt_long (argc, argv, "kvcBFM:o:f:D?", ib_long_options, &optindex)) != -1)
+	while ((c = getopt_long (argc, argv, "kvcBFM:o:f:Db?", ib_long_options, &optindex)) != -1)
 		switch(c)
 		{
+			case 'b':
+				ignore_bar_in_seqnames= 1;
+				break;
 			case 'B':
 				IS_FORCED_ONE_BLOCK =1;
 				break;
@@ -1097,7 +1109,8 @@ int main_buildindex(int argc,char ** argv)
 	*/
 		 SUBREADputs("Usage:");
 		 SUBREADputs("");
-		 SUBREADputs(" ./subread-buildindex [options] -o <basename> {FASTA file1} [FASTA file2] ...");
+		 SUBREADputs(" ./subread-buildindex [options] -o <basename> {FASTA[.gz] file1}\\");
+		 SUBREADputs("      [FASTA[.gz] file2] ...");
 		 SUBREADputs("");
 		 SUBREADputs("Required arguments:");
 		 SUBREADputs("");
@@ -1138,14 +1151,14 @@ int main_buildindex(int argc,char ** argv)
 	print_subread_logo();
 
 	SUBREADputs("");
-	print_in_box(80, 1, 1, "indexBuilder setting");
+	print_in_box(80, 1, 1, "setting");
 	print_in_box(80, 0, 1, "");
 	print_in_box(80, 0, 0, "               Index name : %s", get_short_fname(output_file));
-	print_in_box(80, 0, 0, "              Index space : %s", IS_COLOR_SPACE?"color-space":"base-space");
+	print_in_box(80, 0, 0, "              Index space : %s", IS_COLOR_SPACE?"color space":"base space");
 
 	if(IS_FORCED_ONE_BLOCK)
 	{
-		print_in_box(80, 0, 0, "          One block index : yes");
+	print_in_box(80, 0, 0, "              Index split : ns");
 		memory_limit = GENE_SLIDING_STEP==1?22000:11500;
 	}
 	else
@@ -1159,7 +1172,7 @@ int main_buildindex(int argc,char ** argv)
 			print_in_box(80, 0, 0, "                   Memory : %u Mbytes", memory_limit);
 	}
 	print_in_box(80, 0, 0, "         Repeat threshold : %d repeats", threshold);
-	print_in_box(80, 0, 0, " Distance to next subread : %d", GENE_SLIDING_STEP);
+	print_in_box(80, 0, 0, "             Gapped index : %s", GENE_SLIDING_STEP>1?"yes":"no");
 	print_in_box(80, 0, 0, "");
 	print_in_box(80, 0, 0, "              Input files : %d file%s in total",  argc - optind, (argc - optind>1)?"s":"");
 
@@ -1175,7 +1188,7 @@ int main_buildindex(int argc,char ** argv)
 		print_in_box(94, 0, 0, "                            %c[32m%c%c[36m %s%c[0m", CHAR_ESC, o_char, CHAR_ESC,  get_short_fname(fasta_fn) , CHAR_ESC);
 	}
 	print_in_box(80, 0, 0, "");
-	print_in_box(80, 2, 1, "http://subread.sourceforge.net/");
+	print_in_box(80, 2, 1, "");
 	SUBREADputs("");
 
 	print_in_box(80, 1, 1, "Running");
@@ -1185,16 +1198,12 @@ int main_buildindex(int argc,char ** argv)
 	{
 		char * fasta_fn = *(argv+optind+x1);
 		int f_type = probe_file_type_fast(fasta_fn);
-		if(f_type != FILE_TYPE_FASTA && f_type != FILE_TYPE_NONEXIST){
+		if(f_type != FILE_TYPE_GZIP_FASTA && f_type != FILE_TYPE_FASTA && f_type != FILE_TYPE_NONEXIST){
 			SUBREADprintf("ERROR: '%s' is not a Fasta file.\n", fasta_fn);
-			if(f_type == FILE_TYPE_GZIP_FASTA){
-				SUBREADprintf("The index builder does not accept gzipped Fasta files.\nPlease decompress the gzipped Fasza files before building the index.\n");
-			}
 			STANDALONE_exit(-1);
 		}
 	}
 
-
 	begin_ftime = miltime();
 
 	for(x1 = strlen(output_file); x1 >=0; x1--){
@@ -1226,10 +1235,11 @@ int main_buildindex(int argc,char ** argv)
 
 	if(!ret)
 	{
+		long long actual_bases=0;
 		gehash_t huge_table;
 		gehash_create(& huge_table, 50000000 * (GENE_SLIDING_STEP==1?3:1), 0);
-		ret = ret || scan_gene_index(output_file, ptr_tmp_fa_file , 1, threshold, &huge_table);
-		ret = ret || build_gene_index(output_file, ptr_tmp_fa_file , 1,  memory_limit, threshold, &huge_table, chromosome_lengths);
+		ret = ret || scan_gene_index(output_file, ptr_tmp_fa_file , 1, threshold, &huge_table, &actual_bases);
+		ret = ret || build_gene_index(output_file, ptr_tmp_fa_file , 1,  memory_limit, threshold, &huge_table, chromosome_lengths, actual_bases);
 		if(!ret){
 			print_in_box(80, 0, 1, "Total running time: %.1f minutes.", (miltime()-begin_ftime)/60);
 			print_in_box(89, 0, 1, "Index %c[36m%s%c[0m was successfully built!", CHAR_ESC, output_file, CHAR_ESC);
@@ -1243,7 +1253,7 @@ int main_buildindex(int argc,char ** argv)
 	tmp_fa_file[0]=0;
 
 	print_in_box(80, 0, 0, "");
-	print_in_box(80, 2, 1, "http://subread.sourceforge.net/");
+	print_in_box(80, 2, 1, "");
 	SUBREADputs("");
 	return ret;
 }
