@@ -120,6 +120,8 @@ void set_hash_funcs(HashTable * tab, int is_key_freed, int is_value_string)
 	HashTableSetKeyComparisonFunction(tab,fc_strcmp_chro);
 }
 
+int warning_reported_repeated;
+
 void do_find_common(char ** file_names, int files) 
 {
 	int badfiles = 0;
@@ -194,6 +196,7 @@ void do_find_common(char ** file_names, int files)
 
 			char *tmp_pnt_alt = NULL;
 
+			//SUBREADprintf("GO_INTO_STR: %s\n", alt_str);
 			while(1)
 			{
 				char * alt_one = strtok_r(tmp_pnt_alt?NULL:alt_str,",", &tmp_pnt_alt);
@@ -223,7 +226,8 @@ void do_find_common(char ** file_names, int files)
 					}
 					else
 					{
-						SUBREADprintf("Warning: repeated rows are found in the first input file!\n");
+						if(!warning_reported_repeated)SUBREADprintf("Warning: repeated rows are found in the first input file!\n");
+						warning_reported_repeated=1;
 						free(ky);
 					}
 				}
@@ -249,6 +253,7 @@ void do_find_common(char ** file_names, int files)
 					}
 					free(ky);
 				}
+				if(NULL == tmp_pnt_alt) break;
 			}
 		}
 
@@ -365,6 +370,7 @@ int findCommonVariants(int argc, char ** argv)
 	opterr = 1;
 	optopt = 63;
 	output_file_name[0]=0;
+	warning_reported_repeated=0;
 
 	while((c = getopt_long (argc, argv, "o:h", propm_long_options, &option_index)) != -1)
 	{
