@@ -62,8 +62,8 @@
 
 //#warning "============== REMOVE '* 15' FROM THE NEXT LINE ================"
 #define MAX_READ_LENGTH ( 1210 )
-#define MAX_READ_NAME_LEN 100 
-#define MAX_CHROMOSOME_NAME_LEN 100 
+#define MAX_READ_NAME_LEN 200 
+#define MAX_CHROMOSOME_NAME_LEN 200 
 #define MAX_FILE_NAME_LENGTH 300
 #define FEATURE_NAME_LENGTH 256 
 
@@ -129,9 +129,13 @@ typedef pthread_spinlock_t subread_lock_t;
 #define CORE_SOFT_BR_CHAR '\n'
 #else
 #define STANDALONE_exit(i) return i;
-#define SUBREADprintf Rprintf
-#define SUBREADputs(x) Rprintf("%s\n",(x))
-#define SUBREADputchar(X) Rprintf("%c",(X)) 
+
+int safeRprintf(char *fmt, ...);
+void msgqu_printf(const char * fmt, ...);
+
+#define SUBREADprintf  msgqu_printf
+#define SUBREADputs(x) msgqu_printf("%s\n",(x))
+#define SUBREADputchar(X) msgqu_printf("%c",(X)) 
 #define SUBREADfflush(X) 
 #define CORE_SOFT_BR_CHAR '\n'
 
@@ -428,6 +432,7 @@ typedef struct {
 	int space_type ;
 	int file_type ;
 	void * input_fp;   // can be system (FILE * sam or fastq or fasta), (seekable_zfile_t *)
+	char gzfa_last_name[MAX_READ_NAME_LEN];
 	unsigned long long read_chunk_start;
 } gene_input_t;
 
@@ -436,6 +441,7 @@ typedef struct{
 		unsigned long long simple_file_position;
 		seekable_position_t seekable_gzip_position;
 	};
+	char gzfa_last_name[MAX_READ_NAME_LEN];
 } gene_inputfile_position_t;
 
 
@@ -466,6 +472,7 @@ typedef struct{
 	unsigned short flags;
 	unsigned int read_number;
 	unsigned int pos;
+	unsigned short mapped_segment_in_read;
 	char strand;	// 0 = positive, 1 = negative
 } base_block_temp_read_t;
 
